/*
 Copyright (c) 2019, Apple Inc. All rights reserved.
 
 Redistribution and use in source and binary forms, with or without modification,
 are permitted provided that the following conditions are met:
 
 1.  Redistributions of source code must retain the above copyright notice, this
 list of conditions and the following disclaimer.
 
 2.  Redistributions in binary form must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation and/or
 other materials provided with the distribution.
 
 3. Neither the name of the copyright holder(s) nor the names of any contributors
 may be used to endorse or promote products derived from this software without
 specific prior written permission. No license is granted to the trademarks of
 the copyright holders even if such marks are included in this software.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
 FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

@testable import CareKitStore
import XCTest

class TestCoreDataSchemaWithOutcomes: XCTestCase {
    var store: OCKStore!

    override func setUp() {
        super.setUp()
        store = OCKStore(name: "test", type: .inMemory)
    }

    func testCanSaveOutcomeWithvalues() {
        let scheduleElement = OCKCDScheduleElement(context: store.context)
        scheduleElement.startDate = Date()
        scheduleElement.daysInterval = 1
        scheduleElement.duration = .seconds(42)

        let task1 = OCKCDTask(context: store.context)
        task1.id = "task_1"
        task1.effectiveDate = Date()
        task1.allowsMissingRelationships = true
        task1.scheduleElements = Set([scheduleElement])

        let value1 = OCKCDOutcomeValue(context: store.context)
        value1.kind = "sleep-time"
        value1.dateValue = Date()
        value1.type = .date

        let value2 = OCKCDOutcomeValue(context: store.context)
        value2.kind = "circumference-diameter-ratio"
        value2.doubleValue = 3.14
        value2.type = .double

        let value3 = OCKCDOutcomeValue(context: store.context)
        value3.kind = "favorite-japanese-food"
        value3.integerValue = 8_929
        value3.type = .integer

        let outcome = OCKCDOutcome(context: store.context)
        outcome.taskOccurrenceIndex = 0
        outcome.values = Set([value1, value2, value3])
        outcome.task = task1

        XCTAssertNoThrow(try store.context.save())
        XCTAssert(outcome.values.count == 3)
    }

    func testCannotSaveOutcomeWithoutAssociatingItWithATask() {
        let outcome = OCKCDOutcome(context: store.context)
        outcome.taskOccurrenceIndex = 0
        outcome.values = Set([])

        XCTAssertThrowsError(try store.context.save())
    }
}
